@@ -51,7 +51,8 @@ circ.mean <- function(deg){
 #' vector  components and wind direction and speed for each coordenate
 #' in the study area defined by lon1/lon2 and lat1/lat2.
 #' @note Longitude coordenates are provided by GFS dataset in 0/360 notation
-#' and transformed internaly into -180/180.
+#' and transformed internaly into -180/180. Wind "dir" denotes where the
+#' wind is going (toward), not from where is coming.
 #' @author Javier Fernández-López (jflopez@@rjb.csic.es)
 #' @seealso \code{\link{wind.dl_2}}, \code{\link{wind2raster}}
 #' @references
@@ -433,7 +434,7 @@ wind2raster_int<- function(x){
   ras[] <- x$dir
   ras2[] <- x$speed
   tmp <- stack (ras, ras2)
-  names(tmp)<- c("wind.direction", "wind.speed")
+  names(tmp)<- c("direction", "speed")
   return(tmp)
 }
 
@@ -605,10 +606,10 @@ flow.dispersion_int <- function(stack, fun=cost.FMGS, output="transitionLayer",
 
     output <- match.arg(output, c("raw", "transitionLayer"))
 
-    DL <- as.matrix(stack$wind.direction)
-    SL <- as.matrix(stack$wind.speed)
-    M <- matrix(as.integer(1:ncell(stack$wind.direction)),
-                nrow = nrow(stack$wind.direction), byrow=TRUE)
+    DL <- as.matrix(stack$direction)
+    SL <- as.matrix(stack$speed)
+    M <- matrix(as.integer(1:ncell(stack$direction)),
+                nrow = nrow(stack$direction), byrow=TRUE)
     nr <- nrow(M)
     nc <- ncol(M)
 
@@ -689,7 +690,7 @@ flow.dispersion_int <- function(stack, fun=cost.FMGS, output="transitionLayer",
     tl <- sparseMatrix(i=ii, j=jj, x=xx)
     if(output == "raw") return(tl)
     if(output == "transitionLayer") {
-        tmp <- transition(stack$wind.direction, transitionFunction=function(x) 0, directions=8)
+        tmp <- transition(stack$direction, transitionFunction=function(x) 0, directions=8)
         transitionMatrix(tmp)<-sparseMatrix(i=ii, j=jj, x= 1 / xx)
         return(tmp)
     }
@@ -828,7 +829,7 @@ tidy.rWind_series <- function(x, ...){
 #' @author Javier Fernández-López (jflopez@@rjb.csic.es)
 #' @seealso \code{\link{wind.dl}}
 #' @references https://en.wikipedia.org/wiki/Cross_product
-#' @keywords ~kwd1 ~kwd2
+#' @keywords ~average ~mean
 #' @examples
 #' data(wind.series)
 #' wind_average<- wind.mean(wind.series)
@@ -859,3 +860,250 @@ wind.mean <- function(x){
     return(res)
 }
 
+
+###############################################################################
+# Some new and experimental functions to download OSCAR Sea Surface Velocity data
+# https://coastwatch.pfeg.noaa.gov/erddap/griddap/jplOscar_LonPM180.html
+# https://coastwatch.pfeg.noaa.gov/erddap/info/jplOscar_LonPM180/index.html
+# This is a beta version, please use it carefully
+
+oscar.fit_int <- function (tmpx) {
+  tmpx <- cbind(tmpx[,1],tmpx[,3:6])
+  ###### DIRECTION
+  direction <- atan2(tmpx[,4], tmpx[,5])
+  direction <- rad2deg(direction)
+  dir_filter <- direction[!is.nan(direction)]
+  dir_filter[dir_filter < 0] <- 360 + dir_filter[dir_filter < 0]
+  direction[!is.nan(direction)] <- dir_filter
+  ###### SPEED
+  speed <- sqrt( (tmpx[,4] * tmpx[,4]) + (tmpx[,5] * tmpx[,5]))
+  ######
+  names(tmpx)<- c("time", "lat","lon", "u", "v")
+  res <- cbind(tmpx, dir=direction, speed=speed)
+  res <- res[with(res, order(-lat)), ]
+  res[,1] <- ymd_hms(res[,1], truncated = 3)
+  return(res)
+}
+
+#' OSCAR Sea currents data download
+#'
+#' seaOscar.dl downloads sea currents data from the Ocean Surface Current Analyses Real-time (OSCAR)
+#' (https://coastwatch.pfeg.noaa.gov/erddap/info/jplOscar_LonPM180/index.html).
+#' Geospatial resolution is 0.33 degrees and sea currents are calculated for
+#' 15 m depth. CAUTION: OSCAR database has no data between 0 and 20 longitude
+#' degrees. You can use SCUD databse instead \code{\link{seaScud.dl}}
+#'
+#' The output type is determined by type="csv" or type="read-data". If
+#' type="csv" is selected, the function creates a "sea_yyyy_mm_dd.csv" file
+#' that is downloaded at the work directory. If type="read-data" is selected,
+#' an R object (data.frame) is created.
+#'
+#' @param yyyy Selected year.
+#' @param mm Selected month.
+#' @param dd Selected day.
+#' @param lon1 Western longitude
+#' @param lon2 Eastern longitude
+#' @param lat1 Northern latitude
+#' @param lat2 Southern latitude
+#' @param type Output type. "read-data" is selected by default, creating an R
+#' object. If you choose "csv", seaOscar.dl create a a CSV file in your working
+#' directory named "oscar_yyyy_mm_dd.csv".
+#' @param trace if trace = 1 (by default) track downloaded files
+#' @return "rWind" and "data.frame" class object or .csv file with U and V
+#' vector  components and sea current direction and speed for each coordenate
+#' in the study area defined by lon1/lon2 and lat1/lat2.
+#' @author Javier Fernández-López (jflopez@@rjb.csic.es)
+#' @seealso \code{\link{wind.dl_2}}, \code{\link{wind2raster}}, \code{\link{seaScud.dl}}
+#' @references
+#' http://www.digital-geography.com/cloud-gis-getting-weather-data/#.WDOWmbV1DCL
+#'
+#' https://coastwatch.pfeg.noaa.gov/erddap/info/jplOscar_LonPM180/index.html
+#' @keywords ~currents ~sea
+#' @examples
+#'
+#' # Download sea currents for Galapagos Islands
+#' \dontrun{
+#'
+#' seaOscar.dl(2015,1,1,-93,-88,2,-3)
+#'
+#' }
+
+#' @importFrom utils write.table read.csv download.file
+#' @importFrom lubridate ymd year month day hour
+#' @rdname seaOscar.dl
+#' @export seaOscar.dl
+
+seaOscar.dl <- function(yyyy, mm, dd, lon1, lon2, lat1, lat2, type = "read-data", trace = 1) {
+  type <- match.arg(type, c("read-data", "csv"))
+  mm <- sprintf("%02d", mm)
+  dd <- sprintf("%02d", dd)
+  dt <- ymd(paste(yyyy, mm, dd, sep = "-"))
+  yyyy_c <- year(dt)
+  mm_c <- sprintf("%02d", month(dt))
+  dd_c <- sprintf("%02d", day(dt))
+  #tt_c <- sprintf("%02d", hour(dt))
+  testDate <- paste(yyyy_c, "-", mm_c, "-", dd_c, sep = "")
+  print(testDate)
+  if (trace)
+    print(paste(ymd(paste(yyyy_c, mm_c, dd_c, sep = "-")),
+                "downloading...", sep = " "))
+  tryCatch({
+    as.Date(testDate)
+    url_dir <- paste("https://coastwatch.pfeg.noaa.gov/erddap/griddap/jplOscar_LonPM180.csv?u[(",
+                     yyyy_c,"-",mm_c,"-",dd_c,"T00:00:00Z):1:(",yyyy_c,"-",mm_c,"-",dd_c,
+                     "T00:00:00Z)][(15.0):1:(15.0)][(",lat1,"):1:(", lat2,
+                     ")][(",lon1,"):1:(",lon2, ")],v[(",yyyy_c,"-",mm_c,"-",dd_c,
+                     "T00:00:00Z):1:(",yyyy_c,"-",mm_c,"-",dd_c,"T00:00:00Z)][(15.0):1:(15.0)][(",
+                     lat1,"):1:(",lat2,")][(", lon1, "):1:(", lon2, ")],um[(",
+                     yyyy_c,"-",mm_c,"-", dd_c, "T00:00:00Z):1:(", yyyy_c, "-", mm_c,
+                     "-", dd_c, "T00:00:00Z)][(15.0):1:(15.0)][(", lat1, "):1:(",
+                     lat2,")][(", lon1, "):1:(", lon2, ")],vm[(", yyyy_c, "-", mm_c,
+                     "-", dd_c,"T00:00:00Z):1:(", yyyy_c, "-", mm_c, "-", dd_c,
+                     "T00:00:00Z)][(15.0):1:(15.0)][(", lat1, "):1:(", lat2,
+                     ")][(" , lon1, "):1:(", lon2,")]", sep="")
+
+    tmp <- read.csv(url_dir, header = FALSE, skip = 2,
+                    stringsAsFactors = FALSE)
+    tmp <- oscar.fit_int(tmp)
+    if (type == "csv") {
+      fname <- paste("oscar_", yyyy_c, "_", mm_c, "_",
+                     dd_c, "_", ".csv", sep = "")
+      write.table(tmp, fname, sep = ",", row.names = FALSE,
+                  col.names = TRUE, quote = FALSE)
+    }
+  }, error = function(e) {
+    cat("ERROR: database not found. Please, check server\n                      connection, date or geographical ranges \n")
+  }, warning = function(w) {
+    cat("ERROR: database not found. Please, check server\n                        connection, date or geographical ranges  \n")
+  })
+  class(tmp) <- c("rWind", "data.frame")
+  return(tmp)
+}
+
+
+###############################################################################
+# Some new and experimental functions to download Surface CUrrents from a
+# Diagnostic model (SCUD): Pacific database.
+# https://bluehub.jrc.ec.europa.eu/erddap/info/hawaii_0958_63c0_45d2/index.html
+# https://bluehub.jrc.ec.europa.eu/erddap/griddap/hawaii_0958_63c0_45d2.graph
+# This is a beta version, please use it carefully
+
+scud.fit_int <- function (tmpx) {
+  tmpx[,3] <- tmpx[,3] %% 360
+  tmpx[tmpx[,3]>=180,3] <- tmpx[tmpx[,3]>=180,3] - 360
+
+  #tmpx <- cbind(tmpx[,1],tmpx[,3:6])
+  ###### DIRECTION
+  direction <- atan2(tmpx[,4], tmpx[,5])
+  direction <- rad2deg(direction)
+  dir_filter <- direction[!is.nan(direction)]
+  dir_filter[dir_filter < 0] <- 360 + dir_filter[dir_filter < 0] # COMPROBAR A MANOOOOO!!!!!!!!!!!
+  direction[!is.nan(direction)] <- dir_filter
+  ###### SPEED
+  speed <- sqrt( (tmpx[,4] * tmpx[,4]) + (tmpx[,5] * tmpx[,5]))
+  ######
+  names(tmpx)<- c("time", "lat","lon", "u", "v")
+  res <- cbind(tmpx, dir=direction, speed=speed)
+  res <- res[with(res, order(-lat)), ]
+  res[,1] <- ymd_hms(res[,1], truncated = 3)
+  return(res)
+}
+
+
+#' SCUD Sea currents data download
+#'
+#' seaScud.dl downloads sea current data from the Surface CUrrents from a
+#' Diagnostic model (SCUD): Pacific
+#' (https://bluehub.jrc.ec.europa.eu/erddap/info/hawaii_0958_63c0_45d2/index.html).
+#' Geospatial resolution is 0.25 degrees. Data availability from 2012-03-17 to
+#' current.
+#'
+#' The output type is determined by type="csv" or type="read-data". If
+#' type="csv" is selected, the function creates a "sea_yyyy_mm_dd.csv" file
+#' that is downloaded at the work directory. If type="read-data" is selected,
+#' an R object (data.frame) is created.
+#'
+#' @param yyyy Selected year.
+#' @param mm Selected month.
+#' @param dd Selected day.
+#' @param lon1 Western longitude
+#' @param lon2 Eastern longitude
+#' @param lat1 Southern latitude
+#' @param lat2 Northern latitude
+#' @param type Output type. "read-data" is selected by default, creating an R
+#' object. If you choose "csv", seaOscar.dl create a a CSV file in your working
+#' directory named "seaSCUD_yyyy_mm_dd.csv".
+#' @param trace if trace = 1 (by default) track downloaded files
+#' @return "rWind" and "data.frame" class object or .csv file with U and V
+#' vector  components and sea current direction and speed for each coordenate
+#' in the study area defined by lon1/lon2 and lat1/lat2.
+#' @author Javier Fernández-López (jflopez@@rjb.csic.es)
+#' @seealso \code{\link{seaOscar.dl}}, \code{\link{wind2raster}}
+#' @references
+#' http://www.digital-geography.com/cloud-gis-getting-weather-data/#.WDOWmbV1DCL
+#'
+#' https://bluehub.jrc.ec.europa.eu/erddap/info/hawaii_0958_63c0_45d2/index.html
+#' @keywords ~currents ~sea
+#' @examples
+#'
+#' # Download sea currents for Galapagos Islands
+#' \dontrun{
+#'
+#' seaScud.dl(2016, 11, 1, -94, -88, -3, 2)
+#'
+#' }
+
+#' @importFrom utils write.table read.csv download.file
+#' @importFrom lubridate ymd year month day hour
+#' @rdname seaScud.dl
+#' @export seaScud.dl
+
+seaScud.dl <- function(yyyy, mm, dd, lon1, lon2, lat1, lat2, type="read-data", trace=1){
+
+  type <- match.arg(type, c("read-data", "csv"))
+
+  mm<-sprintf("%02d", mm)
+  dd<-sprintf("%02d", dd)
+
+  # Create a sequence with all dates available between selected dates
+  dt <- ymd(paste(yyyy,mm,dd, sep="-"))
+
+  yyyy_c <- year(dt)
+  mm_c <- sprintf("%02d",month(dt))
+  dd_c <- sprintf("%02d",day(dt))
+
+  testDate <- paste(yyyy_c,"-",mm_c,"-",dd_c, sep="")
+  print(testDate)
+  if(trace)print(paste( ymd(paste(yyyy_c,mm_c,dd_c, sep="-")),
+                        "downloading...", sep= " "))
+
+  tryCatch({
+    as.Date(testDate)
+    if (lon1 < 0){
+      lon1<-360-(abs(lon1))
+    }
+    if (lon2 < 0){
+      lon2<-360-(abs(lon2))
+    }
+    #browser()
+    url_dir<- paste("https://bluehub.jrc.ec.europa.eu/erddap/griddap/hawaii_0958_63c0_45d2.csv?u[(",yyyy_c,"-",mm_c,"-",dd_c,"T00:00:00Z)][(",lat1,"):(",lat2,")][(",lon1,"):(",lon2,")],v[(",yyyy_c,"-",mm_c,"-",dd_c,"T00:00:00Z)][(",lat1,"):(",lat2,")][(",lon1,"):(",lon2,")]&.draw=vectors&.vars=longitude|latitude|u|v&.color=0x000000", sep="")
+    #browser()
+    tmp <- read.csv(url_dir, header=FALSE, skip=2, stringsAsFactors=FALSE)
+    #browser()
+    tmp <- scud.fit_int(tmp)
+    if (type == "csv"){
+      fname <- paste("seaSCUD_",yyyy_c,"_",mm_c,"_",dd_c,".csv", sep="")
+      write.table(tmp, fname, sep = ",", row.names = FALSE,
+                  col.names = TRUE, quote = FALSE)
+    }
+
+  },
+  error=function(e){cat("ERROR: database not found. Please, check server
+                      connection, date or geographical ranges \n")},
+  warning=function(w){cat("ERROR: database not found. Please, check server
+                        connection, date or geographical ranges  \n")}
+  )
+  class(tmp) <-  c("rWind", "data.frame")
+  return(tmp)
+}
+
